@@ -457,10 +457,9 @@ const gint kz_actions_popup_len = G_N_ELEMENTS(kz_actions_popup);
 
 
 GtkActionGroup *
-kz_actions_popup_create_group (KzWindow *kz, GtkAccelGroup *accel_group)
+kz_actions_popup_create_group (KzWindow *kz)
 {
 	GtkActionGroup *action_group;
-	GList *node, *action_list;
 
 	action_group = gtk_action_group_new("KzWindowPopup");
 
@@ -471,16 +470,6 @@ kz_actions_popup_create_group (KzWindow *kz, GtkAccelGroup *accel_group)
 				     kz_actions_popup_len,
 				     kz);
 
-	action_list = gtk_action_group_list_actions(action_group);
-	
-	for (node = action_list; node; node = g_list_next(node))
-	{
-		gtk_action_set_accel_group(GTK_ACTION(node->data),
-					   accel_group);
-		gtk_action_connect_accelerator(GTK_ACTION(node->data));
-	}
-	g_list_free(action_list);
-
 	return action_group;
 }
 
