@@ -54,6 +54,7 @@
 #include "kz-entry.h"
 #include "kz-proxy-menu.h"
 #include "kz-popup-tablist.h"
+#include "kz-ext.h"
 
 extern GtkWidget *kz_moz_embed_new (const gchar *url);
 #ifdef ENABLE_GTK_WEBCORE
@@ -99,14 +100,17 @@ struct _KzWindowPrivate
 	GList *open_hist;
 	GList *view_hist;
 	GNode *tab_tree;
+
+	/* statusbar id */
+	guint status_link_message;
+	guint status_gesture;
+	guint status_search;
 };
 #define KZ_WINDOW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_WINDOW, KzWindowPrivate))
 
-enum {
-	STATUS_LINK_MESSAGE,
-	STATUS_GESTURE,
-	STATUS_SEARCH
-};
+#define STATUS_LINK_MESSAGE "INTERNAL_LINK_MESSAGE"
+#define STATUS_GESTURE "INTERNAL_GESTURE"
+#define STATUS_SEARCH "INTERNAL_SEARCH"
 
 enum {
 	TARGET_KAZEHAKASE_TAB,
@@ -164,7 +168,6 @@ static void     kz_window_remove_tab     (KzWindow      *kz,
 static KzEmbed *kz_window_create_embed   (KzWindow      *kz,
 					  const gchar   *url);
 
-static void     kz_window_set_gesture_items        (KzWindow   *kz);
 static void     kz_window_set_cur_embed_callbacks  (KzWindow   *kz,
 						    KzEmbed    *kzembed);
 static void     kz_window_unset_cur_embed_callbacks(KzWindow   *kz,
@@ -388,6 +391,52 @@ kz_window_class_init (KzWindowClass *klass)
 	g_type_class_add_private (gobject_class, sizeof(KzWindowPrivate));
 }
 
+void
+kz_window_connect_action (KzWindow *kz, GtkAction *action)
+{
+	GtkAccelGroup *accel_group;
+
+	accel_group = gtk_ui_manager_get_accel_group(kz->menu_merge);
+
+	gtk_action_set_accel_group(action, accel_group);
+	gtk_action_connect_accelerator(action);
+}
+
+void
+kz_window_disconnect_action (KzWindow *kz, GtkAction *action)
+{
+	gtk_action_disconnect_accelerator(action);
+	gtk_action_set_accel_group(action, NULL);
+}
+
+static void
+kz_window_setup_action_group (GtkActionGroup *action_group,
+			      GtkAccelGroup *accel_group)
+{
+	GList *node, *action_list;
+
+	action_list = gtk_action_group_list_actions(action_group);
+
+	for (node = action_list; node; node = g_list_next(node))
+	{
+		GtkAction *action = GTK_ACTION(node->data);
+		gtk_action_set_accel_group(action, accel_group);
+		gtk_action_connect_accelerator(action);
+	}
+	g_list_free(action_list);
+}
+
+static void
+kz_window_setup_action_groups (KzWindow *kz)
+{
+	GtkAccelGroup *accel_group;
+
+	accel_group = gtk_ui_manager_get_accel_group(kz->menu_merge);
+
+	kz_window_setup_action_group (kz->actions, accel_group);
+	kz_window_setup_action_group (kz->popup_actions, accel_group);
+	kz_window_setup_action_group (kz->tabpop_actions, accel_group);
+}
 
 static void
 kz_window_init (KzWindow *kz)
@@ -396,7 +445,6 @@ kz_window_init (KzWindow *kz)
 	GtkSettings *setting;
 	GtkWidget *vseparator;
 	GList *children, *node;
-	GtkAccelGroup *accel_group;
 	GtkSettings *settings;
 	KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE (kz);
 
@@ -428,11 +476,10 @@ kz_window_init (KzWindow *kz)
 	kz->feed_info          = g_object_ref(kz_feed_info_new(kz));
 
 	kz->menu_merge         = gtk_ui_manager_new();
-	accel_group            = gtk_ui_manager_get_accel_group(kz->menu_merge);
 
-	kz->actions            = kz_actions_create_group(kz, accel_group);
-	kz->popup_actions      = kz_actions_popup_create_group(kz, accel_group);
-	kz->tabpop_actions     = kz_actions_tab_popup_create_group(kz, accel_group);
+	kz->actions            = kz_actions_create_group(kz);
+	kz->popup_actions      = kz_actions_popup_create_group(kz);
+	kz->tabpop_actions     = kz_actions_tab_popup_create_group(kz);
 	
 	kz->kzfav              = kz_favicon_get_instance();
 	kz->popup              = kz_popup_preview_get_instance();
@@ -460,15 +507,29 @@ kz_window_init (KzWindow *kz)
 	priv->view_hist    = NULL;
 	priv->tab_tree     = NULL;
 
+	priv->status_link_message =
+		gtk_statusbar_get_context_id(GTK_STATUSBAR(kz->statusbar),
+					     STATUS_LINK_MESSAGE);
+	priv->status_gesture =
+		gtk_statusbar_get_context_id(GTK_STATUSBAR(kz->statusbar),
+					     STATUS_GESTURE);
+	priv->status_search =
+		gtk_statusbar_get_context_id(GTK_STATUSBAR(kz->statusbar),
+					     STATUS_SEARCH);
+
 	kz->tabs   = kz_bookmark_pure_folder_new();
 	kz->closed_tabs     = kz_bookmark_pure_folder_new();
 	kz->history_search  = kz_bookmark_pure_folder_new();
 	
 	kz->is_closing_all  = FALSE;
 
+	kz_ext_setup(kz);
+
+	kz_window_setup_action_groups(kz);
+
 	/* gesture */
 	{
-		kz_window_set_gesture_items(kz);
+		kz_window_update_gesture_items(kz);
 		g_signal_connect(priv->gesture,
 				 "stack_motion",
 				 G_CALLBACK(cb_gesture_stack_motion), kz);
@@ -489,6 +550,8 @@ kz_window_init (KzWindow *kz)
 
 	/* menu & toolbar */
 	{
+		GtkAccelGroup *accel_group;
+
 		kz->menu_box = menu_box = gtk_vbox_new (FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(kz->top_vbox), menu_box,
 				   FALSE, FALSE, 0);   
@@ -504,9 +567,9 @@ kz_window_init (KzWindow *kz)
 		g_signal_connect(kz->menu_merge, "add_widget",
 				 G_CALLBACK(cb_menu_merge_add_widget),
 				 menu_box);
-		gtk_window_add_accel_group(GTK_WINDOW(kz), 
-					   gtk_ui_manager_get_accel_group(kz->menu_merge));
-	
+		accel_group = gtk_ui_manager_get_accel_group(kz->menu_merge);
+		gtk_window_add_accel_group(GTK_WINDOW(kz), accel_group);
+
 		kz_window_sync_ui_level(kz);
 		gtk_ui_manager_ensure_update(kz->menu_merge);
 		g_signal_connect(kz_global_profile,
@@ -1222,6 +1285,9 @@ kz_window_close_tab (KzWindow *kz, GtkWidget *widget)
 CLOSE_TAB:
 	g_free(ret_page);
 	gtk_widget_destroy(widget);
+
+	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(kz->notebook)) == 0)
+		gtk_window_set_title(GTK_WINDOW(kz), _("Kazehakase"));
 }
 
 
@@ -1643,8 +1709,8 @@ kz_window_destroy (GtkObject *object)
 }
 
 
-static void
-kz_window_set_gesture_items (KzWindow *kz)
+void
+kz_window_update_gesture_items (KzWindow *kz)
 {
 	GtkAction *action;
 	KzGestureItems *items;
@@ -2214,7 +2280,7 @@ cb_profile_gesture_changed (KzProfile *profile,
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	kz_window_set_gesture_items(kz);
+	kz_window_update_gesture_items(kz);
 }
 
 
@@ -2497,9 +2563,12 @@ search_found(GtkWidget* widget, KzWindow *kz)
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	if(kz->statusbar)
+	if(kz->statusbar) {
+		KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE (kz);
 		gtk_statusbar_pop(GTK_STATUSBAR(kz->statusbar),
-				  STATUS_SEARCH);
+				  priv->status_search);
+	}
+
 	gtk_widget_modify_base(widget, GTK_STATE_NORMAL, &white);
 	gtk_widget_modify_text(widget, GTK_STATE_NORMAL, &black);
 }
@@ -2518,10 +2587,11 @@ search_not_found(GtkWidget* widget, KzWindow *kz)
 	message = g_strdup_printf(_("%s not found"), search_word);
 	if(kz->statusbar) 
 	{
+		KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE (kz);
 		gtk_statusbar_pop(GTK_STATUSBAR(kz->statusbar),
-				  STATUS_SEARCH);
+				  priv->status_search);
 		gtk_statusbar_push(GTK_STATUSBAR(kz->statusbar),
-				   STATUS_SEARCH,
+				   priv->status_search,
 				   message);
 	}
 	g_free(message);
@@ -2640,6 +2710,7 @@ cb_gesture_stack_motion (KzGesture *gesture, KzGestureMotion motion,
 {
 	const gchar *label;
 	gchar buf1[256], buf2[256];
+	KzWindowPrivate *priv;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
@@ -2652,9 +2723,10 @@ cb_gesture_stack_motion (KzGesture *gesture, KzGestureMotion motion,
 		g_snprintf(buf2, G_N_ELEMENTS(buf2),
 			   _("Gesture: %s"), buf1);
 
-	gtk_statusbar_pop(GTK_STATUSBAR(kz->statusbar), STATUS_GESTURE);
+	priv = KZ_WINDOW_GET_PRIVATE (kz);
+	gtk_statusbar_pop(GTK_STATUSBAR(kz->statusbar), priv->status_gesture);
 	gtk_statusbar_push(GTK_STATUSBAR(kz->statusbar),
-			   STATUS_GESTURE, buf2);
+			   priv->status_gesture, buf2);
 }
 
 
@@ -2774,10 +2846,11 @@ cb_embed_link_message (KzEmbed *embed, KzWindow *kz)
 	newMessage = kz_embed_get_link_message(KZ_EMBED(embed));
 	if(kz->statusbar) 
 	{
+		KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE (kz);
 		gtk_statusbar_pop(GTK_STATUSBAR(kz->statusbar),
-				  STATUS_LINK_MESSAGE);
+				  priv->status_link_message);
 		gtk_statusbar_push(GTK_STATUSBAR(kz->statusbar),
-				   STATUS_LINK_MESSAGE, newMessage);
+				   priv->status_link_message, newMessage);
 	}
 
 	g_free(newMessage);
@@ -2836,7 +2909,8 @@ cb_embed_close_tab (GtkObject *obj, KzWindow *kz)
 	priv->view_hist = g_list_remove(priv->view_hist, kzembed);
 	priv->open_hist = g_list_remove(priv->open_hist, kzembed);
 
-	gtk_statusbar_pop(GTK_STATUSBAR(kz->statusbar), STATUS_LINK_MESSAGE);
+	gtk_statusbar_pop(GTK_STATUSBAR(kz->statusbar),
+			  priv->status_link_message);
 
 	node = g_node_find(priv->tab_tree,
 			   G_IN_ORDER, G_TRAVERSE_ALL, kzembed);
@@ -3105,7 +3179,7 @@ cb_embed_dom_mouse_down (KzEmbed *kzembed, KzEmbedEventMouse *event,
 				  GDK_BUTTON_PRESS_MASK,
 				  NULL, cursor, gtk_get_current_event_time ());
 		gtk_statusbar_push(GTK_STATUSBAR(kz->statusbar),
-				   STATUS_GESTURE, _("Gesture:"));
+				   priv->status_gesture, _("Gesture:"));
 		break;
 	}
 	default:
@@ -3202,7 +3276,7 @@ cb_window_motion_notify_event (GtkWidget *widget,
 			}
 			kz_gesture_perform(priv->gesture);
 			gtk_statusbar_push(GTK_STATUSBAR(KZ_WINDOW(widget)->statusbar),
-				   STATUS_GESTURE, "");
+				   priv->status_gesture, "");
 			priv->start_x = 0;
 			priv->start_y = 0;
 			priv->is_gesture = FALSE;
@@ -3246,7 +3320,7 @@ cb_window_button_release_event (GtkWidget *widget,
 		else
 			kz_gesture_cancel(priv->gesture);
 		gtk_statusbar_pop(GTK_STATUSBAR(KZ_WINDOW(widget)->statusbar),
-				  STATUS_GESTURE);
+				  priv->status_gesture);
 	}
 
 	if (!priv->is_gesture &&
@@ -3385,7 +3459,7 @@ cb_focus_out_event            (GtkWidget       *widget,
 				gdk_pointer_ungrab (gtk_get_current_event_time ());
 			}
 			gtk_statusbar_pop(GTK_STATUSBAR(KZ_WINDOW(widget)->statusbar),
-					  STATUS_GESTURE);
+					  priv->status_gesture);
 		}
 	}
 	return FALSE;
