@@ -22,6 +22,10 @@
  * Contributor(s):
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #define MOZILLA_INTERNAL_API
 #include <nsIServiceManager.h>
 #undef MOZILLA_INTERNAL_API
@@ -32,8 +36,9 @@
 
 #include <nsIDirectoryService.h>
 #include <nsDirectoryServiceDefs.h>
+#include <nsXPCOM.h>
 
-#include "nsEmbedAPI.h"
+#include <nsEmbedAPI.h>
 
 static nsIServiceManager *sServiceManager = nsnull;
 static PRBool             sRegistryInitializedFlag = PR_FALSE;
@@ -69,8 +74,16 @@ static XPCOMCleanupHack sXPCOMCleanupHack;
 #endif
 
 
+#ifdef HAVE_GECKO_1_8
+NS_METHOD
+NS_InitEmbedding(nsILocalFile *mozBinDirectory,
+                 nsIDirectoryServiceProvider *appFileLocProvider,
+                 nsStaticModuleInfo const *aStaticComponents,
+                 PRUint32 aStaticComponentCount)
+#else
 nsresult NS_InitEmbedding(nsILocalFile *mozBinDirectory,
                           nsIDirectoryServiceProvider *appFileLocProvider)
+#endif
 {
     nsresult rv;
 
@@ -86,7 +99,12 @@ nsresult NS_InitEmbedding(nsILocalFile *mozBinDirectory,
 #endif
     {
         // Initialise XPCOM
+#ifdef HAVE_GECKO_1_8
+        rv = NS_InitXPCOM3(&sServiceManager, mozBinDirectory, appFileLocProvider,
+                           aStaticComponents, aStaticComponentCount);
+#else
         rv = NS_InitXPCOM2(&sServiceManager, mozBinDirectory, appFileLocProvider);
+#endif
         NS_ENSURE_SUCCESS(rv, rv);
                 
 #ifdef HACK_AROUND_NONREENTRANT_INITXPCOM
@@ -97,51 +115,58 @@ nsresult NS_InitEmbedding(nsILocalFile *mozBinDirectory,
     // Register components
     if (!sRegistryInitializedFlag)
     {
+#ifdef HAVE_GECKO_1_8
+        nsIComponentRegistrar *registrar;
+        rv = sServiceManager->QueryInterface(NS_GET_IID(nsIComponentRegistrar),
+                                             (void **) &registrar);
+#else
         nsCOMPtr<nsIComponentRegistrar> registrar = do_QueryInterface(sServiceManager, &rv);
-        if (NS_FAILED(rv))
-        {
-            NS_WARNING("Could not QI to registrar");
-            return rv;
-        }
-#ifdef DEBUG
-        rv = registrar->AutoRegister(nsnull);
-
-        if (NS_FAILED(rv))
-        {
-            NS_WARNING("Could not AutoRegister");
-            return rv;
-        }
-
-        // If the application is using an GRE, then, 
-        // auto register components in the GRE directory as well.
-        //
-        // The application indicates that it's using an GRE by
-        // returning a valid nsIFile when queried (via appFileLocProvider)
-        // for the NS_GRE_DIR atom as shown below
-        //
-        if (appFileLocProvider)
-        {
-            nsCOMPtr<nsIFile> greDir;
-            PRBool persistent = PR_TRUE;
-
-            appFileLocProvider->GetFile(NS_GRE_DIR, &persistent, getter_AddRefs(greDir));
-
-            if (greDir)
-            {
-                rv = registrar->AutoRegister(greDir);
-                NS_ASSERTION(NS_SUCCEEDED(rv), "Could not AutoRegister GRE components");
-            }
-        }
 #endif
         sRegistryInitializedFlag = PR_TRUE;
     }
 
+#ifdef HAVE_GECKO_1_8
+    nsIComponentManager *compMgr;
+    rv = sServiceManager->QueryInterface(NS_GET_IID(nsIComponentManager),
+                                         (void **) &compMgr);
+    if (NS_FAILED(rv))
+        return rv;
+
+    nsIObserver *startupNotifier;
+    rv = compMgr->CreateInstanceByContractID(NS_APPSTARTUPNOTIFIER_CONTRACTID,
+                                             NULL,
+                                             NS_GET_IID(nsIObserver),
+                                             (void **) &startupNotifier);
+    NS_RELEASE(compMgr);
+    if (NS_FAILED(rv))
+        return rv;
+
+	  startupNotifier->Observe(nsnull, APPSTARTUP_TOPIC, nsnull);
+    NS_RELEASE(startupNotifier);
+
+#else
 	nsCOMPtr<nsIObserver> mStartupNotifier = do_CreateInstance(NS_APPSTARTUPNOTIFIER_CONTRACTID, &rv);
 	if(NS_FAILED(rv))
 		return rv;
 	mStartupNotifier->Observe(nsnull, APPSTARTUP_TOPIC, nsnull);
+#endif
 
 #ifdef HACK_AROUND_THREADING_ISSUES
+#ifdef HAVE_GECKO_1_8
+    // XXX force certain objects to be created on the main thread
+    nsIStringBundleService *bundleService;
+    rv = sServiceManager->GetServiceByContractID(NS_STRINGBUNDLE_CONTRACTID,
+                                                 NS_GET_IID(nsIStringBundleService),
+                                                 (void **) &bundleService);
+    if (NS_SUCCEEDED(rv))
+    {
+        nsIStringBundle *stringBundle;
+        const char propertyURL[] = "chrome://necko/locale/necko.properties";
+        rv = bundleService->CreateBundle(propertyURL, &stringBundle);
+        NS_RELEASE(stringBundle);
+        NS_RELEASE(bundleService);
+    }
+#else
     // XXX force certain objects to be created on the main thread
     nsCOMPtr<nsIStringBundleService> sBundleService;
     sBundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
@@ -152,13 +177,19 @@ nsresult NS_InitEmbedding(nsILocalFile *mozBinDirectory,
         rv = sBundleService->CreateBundle(propertyURL,
                                           getter_AddRefs(stringBundle));
     }
+#endif
 #endif
 
     return NS_OK;
 
 }
 
+#ifdef HAVE_GECKO_1_8
+NS_METHOD
+NS_TermEmbedding()
+#else
 nsresult NS_TermEmbedding()
+#endif
 {
     // Reentrant calls to this method do nothing except decrement a counter
     if (sInitCounter > 1)
